/* -----------------------------------------------------------------------------
*
*  npmysql.c
*
*  ANTz - realtime 3D data visualization tools for the real-world, based on NPE.
*
*  ANTz is hosted at http://openantz.com and NPE at http://neuralphysics.org
*
*  Written in 2010-2012 by Shane Saxon - contact@saxondigital.net
*
*  Please see main.c for a complete list of additional code contributors.
*
*  To the extent possible under law, the author(s) have dedicated all copyright 
*  and related and neighboring rights to this software to the public domain
*  worldwide. This software is distributed without any warranty.
*
*  Released under the CC0 license, which is GPL compatible.
*
*  You should have received a copy of the CC0 Public Domain Dedication along
*  with this software (license file named COPYING.txt). If not, see
*  http://creativecommons.org/publicdomain/zero/1.0/
*
* --------------------------------------------------------------------------- */

#include "npmysql.h"

enum {
	WHERE = 0,
	ORDERBY = 1,
};

// Returns 0 if successful
int npFreeMysqlObject(struct dbConnect *connect, struct mysqlObject *object, void* dataRef)
{
	switch(object->type)
	{
	case TABLE:
		npFreeMysqlTable(connect, object, dataRef);
		break;
	case DATABASE:
		npFreeMysqlDatabase(connect, object, dataRef);
		break;
	case QUERY:
		npFreeMysqlQuery(connect, object, dataRef);
		break;
	}
	return 0;
}


int npFreeMysqlTable(struct dbConnect *connect, struct mysqlObject *object, void* dataRef)
{
//	int i = 0;
	struct mysqlTable *tmpTable;
	tmpTable = ((struct mysqlTable*)object->myobject);

	printf("\nfree table : %s", tmpTable->name);
//	free(tmpTable->name);
//	free(tmpTable->fields);

//	if(tmpTable->formattedValues != NULL)
//		free(tmpTable->formattedValues);

	free(object);

	return 0;
}

int npFreeMysqlDatabase(struct dbConnect *connect,struct mysqlObject *object, void* dataRef)
{
	struct mysqlDatabase *tmpDatabase;
	tmpDatabase = ((struct mysqlDatabase*)object->myobject);
	
	free(tmpDatabase->name);

	free(tmpDatabase);
	free(object);
	return 0;
}

int npFreeMysqlQuery(struct dbConnect *connect, struct mysqlObject *object, void* dataRef)
{
	int i = 0;
	struct mysqlQuery *tmpQuery;
	tmpQuery = ((struct mysqlQuery*)object->myobject);
//	free(tmpQuery->subject);

	for(i = 0; i < tmpQuery->numOfValues; i++)
		free(tmpQuery->values[i]);

	if(tmpQuery->values != NULL)
		free(tmpQuery->values);

	printf("numOfResults B : %d", tmpQuery->numOfResults);
	for(i = 0; i < tmpQuery->numOfResults; i++)
	{
		printf("\nI:%d", i);
		if(tmpQuery->result[i] != NULL)
		{
			printf("\nNOT NULL");
			//mysql_free_result(tmpQuery->result[i]); // debug db // Sometimes mysql_query return null even on success
		}
	}

//		free(tmpQuery->result[0]);

	free(tmpQuery);
	free(object);
	return 0;
}

// This should have more parameters for engines and whatnot, debug ll
int npCreateAntzDatabaseMysql(struct dbConnect *connect, void *dataRef)
{

	npCreateDatabaseMysql(connect, "Antz", dataRef);

	return 0;
}

int npUseDatabaseMysql(struct dbConnect *connect, char *name, void* dataRef)
{
	struct mysqlObject *query = npInitMysqlQuery();
	struct mysqlObject *database = npInitMySQLDatabase(name);

	npSetMysqlQueryObject(connect, query, database, dataRef);
	if(npSetMysqlQueryCommand(connect, query, mysqlUSE, dataRef) == 0)
	{
		npGenMysqlQuery(query, dataRef);
	//	npNewExecuteMysqlQuery(query, connect);
		npNewExecuteMysqlQuery(connect, query, dataRef);
	//	connect->db = name;
		connect->db = name;
	}
	npFreeMysqlObject(connect, query, dataRef);
	npFreeMysqlObject(connect, database, dataRef);
	return 0;
}

int	npUseAntzDatabaseMysql(struct dbConnect *connect, void* dataRef)
{
//	pData data = (pData) npGetDataRef();
	npUseDatabaseMysql(connect, "Antz", dataRef);
//	npUseDatabaseMysql(connect, "Antz");

	return 0;
}

int npCreateDatabaseMysql(struct dbConnect *connect, char *name, void* dataRef)
{
	struct mysqlObject *query = npInitMysqlQuery();
	struct mysqlObject *database = npInitMySQLDatabase(name);

	npSetMysqlQueryObject(connect, query, database, dataRef);
	if(npSetMysqlQueryCommand(connect, query, mysqlCREATE, dataRef) == 0)
	{
		npGenMysqlQuery(query, dataRef);
		npNewExecuteMysqlQuery(connect, query, dataRef);
	}

	npFreeMysqlObject(connect, query, dataRef);
	npFreeMysqlObject(connect, database, dataRef);

	return 0;
}



int npCreateTableMysql(struct dbConnect *connect, char *name, char *fields, void* dataRef)
{
	struct mysqlObject *query = npInitMysqlQuery();
	struct mysqlObject *table = npNewInitMySQLTable(name);

	npSetMysqlQueryObject(connect, query, table, dataRef);
	npSetMysqlQueryCommand(connect, query, mysqlCREATE, dataRef);
	printf("\nnodeFields : %s\n", fields);
	npSetMysqlQueryFields(connect, query, fields);
	npGenMysqlQuery(query, dataRef);
	npNewExecuteMysqlQuery(connect, query, dataRef);
	
	npFreeMysqlObject(connect, query, dataRef);
	npFreeMysqlObject(connect, table, dataRef);
	
	return 0;
}

int	npCreateAntzNodeStateTable(struct dbConnect *connect, void* dataRef)
{
	char* nodeFields = malloc(2000);

	npGenMysqlFields(nodeFields, kNPmapFileBufferMax, kNPnode, dataRef);
	npCreateTableMysql(connect, "nodeStateTable", nodeFields, dataRef); // Get return value on success or failure

	free(nodeFields);

	return 0;
}

int npCreateAntzTagsTable(struct dbConnect *connect, void* dataRef)
{
	pData data = (pData) npGetDataRef();

	char* tagFields = malloc(2000);

	npGenMysqlFields(tagFields, kNPmapFileBufferMax, kNPtag, data);
	npCreateTableMysql(connect, "tagsTable", tagFields, dataRef);

	free(tagFields);
	return 0;
}

int npInsertIntoTableMysql(struct dbConnect *connect, char *name, char **values, int numChunks, void* dataRef)
{
 	struct mysqlObject *query = npInitMysqlQuery();
	struct mysqlObject *table = npNewInitMySQLTable(name);

	npSetMysqlQueryObject(connect, query, table, dataRef);
	npSetMysqlQueryCommand(connect, query, mysqlINSERT, dataRef);
	npSetMysqlQueryValues(connect, query, values, numChunks);
	npGenMysqlQuery(query, dataRef);
	npNewExecuteMysqlQuery(connect, query, dataRef);

	npFreeMysqlObject(connect, query, dataRef);
	npFreeMysqlObject(connect, table, dataRef);
	
	return 0;
}

int npInsertAntzTags(struct dbConnect *connect, void* dataRef)
{	
	int i = 0;
	int numberOfTagChunks;
	char** tagsBuffer = NULL;

	tagsBuffer = npGetRetrieveTextTagsValues(tagsBuffer, &numberOfTagChunks, 0, dataRef); // Generalize (npGetValues(npNodes(npTags, etc)), debug ll
	if(tagsBuffer != NULL)
	{
	tagsBuffer = npFinalFormatChunks(tagsBuffer, &numberOfTagChunks);
	printf("\nnumberOfTagChunks %d", numberOfTagChunks);
	npInsertIntoTableMysql(connect, "tagsTable", tagsBuffer, numberOfTagChunks, dataRef);
	}
		
	// Free tagsBuffer, debug ll
//	for(i = 0; i < numberOfTagChunks; i++)
//	{
//		printf("\nfreeing tagsBuffer[%d]", i);
//		free(tagsBuffer[0]); // debug db port
//	}
	
//	free(tagsBuffer); // debug db port
	
	
	return 0;
}

int npInsertAntzNodeStates(struct dbConnect *connect, void* dataRef)
{
	int numberOfChunks;
	char** buffer = NULL;

	buffer = npGetNodesValues(buffer, &numberOfChunks, dataRef);
	if(buffer != NULL)
	{
		buffer = npFinalFormatChunks(buffer, &numberOfChunks);	
		npInsertIntoTableMysql(connect, "nodeStateTable", buffer, numberOfChunks, dataRef);
	}
		
	return 0;
}

int npDropTableMysql(struct dbConnect *connect,char *name, void* dataRef)
{
	struct mysqlObject *query = npInitMysqlQuery();
	struct mysqlObject *table = npNewInitMySQLTable(name);

	npSetMysqlQueryObject(connect, query, table, dataRef);
	npSetMysqlQueryCommand(connect, query, mysqlDROP, dataRef);
	npGenMysqlQuery(query, dataRef);
	npNewExecuteMysqlQuery(connect, query, dataRef);

	npFreeMysqlObject(connect, query, dataRef);
	npFreeMysqlObject(connect, table, dataRef);

	return 0;
}

int npDropAntzTagsTable(struct dbConnect *connect, void* dataRef)
{
	npDropTableMysql(connect, "tagsTable", dataRef);
	return 0;
}

int npDropAntzNodeStateTable(struct dbConnect *connect, void* dataRef)
{
	npDropTableMysql(connect, "nodeStateTable", dataRef);
	return 0;
}

int npSelectAntzNodeStateTable(struct dbConnect *connect, void* dataRef)
{
	struct mysqlObject *query = npInitMysqlQuery();
	
	npSelectTableMysql(connect, query, "nodeStateTable", dataRef);
	npLoadMysqlResultsIntoAntz(connect, query, 1, dataRef);
	
	npFreeMysqlObject(connect, query, dataRef);
	
	return 0;
}

int npSelectAntzTagsTable(struct dbConnect *connect, void* dataRef)
{
	struct mysqlObject *query = npInitMysqlQuery();
	
	npSelectTableMysql(connect, query, "tagsTable", dataRef);
	npLoadMysqlResultsIntoAntz(connect, query, 0, dataRef);
	
	npFreeMysqlObject(connect, query, dataRef);
	
	return 0;
}

enum {
	NODE_STATE_TABLE = 0,
	TAGS_TABLE = 1,
	CHMAP_TABLE = 2,
	CHANNELS_TABLE = 3
};

enum {
	Create = 0,
	Insert = 1,
	Select = 2,
	Use = 3,
	Drop = 4
};

int npSelectAntzTable(struct dbConnect *connect, int tableType, void* dataRef)
{
	// Error checking, debug ll
	switch(tableType)
	{
	case NODE_STATE_TABLE:
		npSelectAntzNodeStateTable(connect, dataRef);
		break;
	case TAGS_TABLE:
		npSelectAntzTagsTable(connect, dataRef);
		break;
	}

	return 0;
}

int npSelectTableMysql(struct dbConnect *connect, struct mysqlObject *query, char *name, void* dataRef)
{
//	struct mysqlObject *query = npInitMysqlQuery();
	struct mysqlObject *table = npNewInitMySQLTable(name);

	npSetMysqlQueryObject(connect, query, table, dataRef);
	printf("\nQuery focus : %d", ((struct mysqlQuery*)query->myobject)->focus->type);
	if(npSetMysqlQueryCommand(connect, query, mysqlSELECT, dataRef) == 0)
	{
		//npSetMysqlQueryConditionals(tagsSelectQuery, WHERE, "id > 10");
		npGenMysqlQuery(query, dataRef);
		npNewExecuteMysqlQuery(connect, query, dataRef);
		//npLoadMysqlResultsIntoAntz(query, 1, dataRef);
		//npLoadMysqlResultsIntoAntz(tagsSelectQuery, 0, dataRef); //enum??
	}
	//npFreeMysqlQuery(tagsSelectQuery);
//	npFreeMysqlObject(query);
	npFreeMysqlObject(connect, table, dataRef);
	return 0;
}

int npSelectAntzChMapTable(struct dbConnect *connect, void* dataRef)
{
	struct mysqlObject *query = npInitMysqlQuery();
	
	npSelectTableMysql(connect, query, "chMapTable", dataRef);
	npLoadMysqlResultsIntoAntz(connect, query, 1, dataRef);
	
	npFreeMysqlObject(connect, query, dataRef);
	
	return 0;
}

int npSelectAntzChannelsTable(struct dbConnect *connect, void* dataRef)
{
	struct mysqlObject *query = npInitMysqlQuery();
	
	npSelectTableMysql(connect, query, "channelsTable", dataRef);
	npLoadMysqlResultsIntoAntz(query, 1, dataRef, dataRef);
	
	npFreeMysqlObject(connect, query, dataRef);
	
	return 0;
}

int npSelectAntzTableMysql(struct dbConnect *connect, int tableType, void* dataRef)
{
	// Error checking, debug ll
	switch(tableType)
	{
	case NODE_STATE_TABLE:
		npSelectAntzNodeStateTable(connect, dataRef);
		break;
	case TAGS_TABLE:
		npSelectAntzTagsTable(connect, dataRef);
		break;
	case CHMAP_TABLE:
		npSelectAntzChMapTable(connect, dataRef);
		break;
	case CHANNELS_TABLE:
		npSelectAntzChannelsTable(connect, dataRef);
		break;
	}

	return 0;
}

int npDropAntzChMapTable(struct dbConnect *connect, void* dataRef)
{
	npDropTableMysql(connect, "chMapTable", dataRef);
	return 0;
}

int npDropAntzTableMysql(struct dbConnect *connect, int tableType, void* dataRef)
{

	// Error checking, debug ll
	switch(tableType)
	{
	case NODE_STATE_TABLE:
		npDropAntzNodeStateTable(connect, dataRef);
		break;
	case TAGS_TABLE:
		npDropAntzTagsTable(connect, dataRef);
		break;
	case CHMAP_TABLE:
		npDropAntzChMapTable(connect, dataRef);
		break;
	}

	return 0;
}

int npInsertAntzChMap(struct dbConnect *connect, void* dataRef)
{
	int numberOfChunks;
	char** buffer = NULL;
	
	buffer = npRetrieveChMapValues(buffer, &numberOfChunks, dataRef); // Generalize (npGetValues(npNodes(npTags, etc)), debug ll
	
	if(buffer != NULL)
	{
		buffer = npFinalFormatChunks(buffer, &numberOfChunks);
		printf("\nnumberOfChunks %d", numberOfChunks);
		npInsertIntoTableMysql(connect, "ChMapTable", buffer, numberOfChunks, dataRef);
	}
	
	// Free buffer, debug ll
	
	return 0;
}

// Build out, debug ll
char** npRetrieveChannelsValues(struct dbConnect *connect, char** buffer, int* numberOfChunks, void* dataRef)
{
	int i = 0;
	pData data = (pData) dataRef;
	printf("\nnumber of tracks : %d", data->io.ch.numberTracks);
	
	for(i = 0; i < data->io.ch.numberTracks; i++)
	{
		printf("\nMapped track offset %d to attribute %s with channel number %d",
			   data->io.ch.xref[i].trackOffset,
			   data->io.ch.xref[i].attributeName,
			   data->io.ch.xref[i].channelNumber);
	}
	printf("\n\n");
//	readTrackNames(dataRef);
	//pause();
	/*
	data->io.ch.xref[data->io.ch.xrefIndex].attributeName = 
	npNewStrcpy(*(tokens + kNPattributeNameOffset), data);
	data->io.ch.xref[data->io.ch.xrefIndex].trackOffset = 
	atoi(*(tokens + kNPtrackOffset));
	data->io.ch.xref[data->io.ch.xrefIndex].channelNumber = atoi(*(tokens + kNPchannelOffset));
	
	printf("Mapped track offset %d to attribute %s with channel number %d\n",
		   data->io.ch.xref[data->io.ch.xrefIndex].trackOffset,
		   data->io.ch.xref[data->io.ch.xrefIndex].attributeName,
		   data->io.ch.xref[data->io.ch.xrefIndex].channelNumber);
	
	if (data->io.ch.xrefIndex < kNPmaxTrackToAttributeMappings)
		data->io.ch.xrefIndex++;
	 */
	
	return NULL; //for now return null, debug ll
}

int npInsertAntzChannelsTable(struct dbConnect *connect, void* dataRef)
{
	int numberOfChunks;
	char** buffer = NULL;
	
	buffer = npRetrieveChannelsValues(connect, buffer, &numberOfChunks, dataRef); // Generalize, debug ll
	if(buffer != NULL)
	{
		buffer = npFinalFormatChunks(buffer, &numberOfChunks);
		printf("\nnumberOfChunks %d", numberOfChunks);
		npInsertIntoTableMysql(connect, "channelsTable", buffer, numberOfChunks, dataRef);
	}
	
	// free buffer, debug ll
	
	return 0;
}

int npInsertAntzTableMysql(struct dbConnect *connect, int tableType, void* dataRef)
{
	// Error checking, debug ll
	switch(tableType)
	{
	case NODE_STATE_TABLE:
		npInsertAntzNodeStates(connect, dataRef);
		break;
	case TAGS_TABLE:
		npInsertAntzTags(connect, dataRef);
		break;
	case CHMAP_TABLE:
		npInsertAntzChMap(connect, dataRef);
		break;
	case CHANNELS_TABLE:
		npInsertAntzChannelsTable(connect, dataRef);
		break;
	}

	return 0;
}

int npCreateAntzChMapTable(struct dbConnect *connect, void* dataRef)
{
	char* fields = malloc(2000);
	
	npGenMysqlFields(fields, kNPmapFileBufferMax, kNPchMap, dataRef);
	npCreateTableMysql(connect, "chMapTable", fields, dataRef);
	
	free(fields);
	
	return 0;
}

int npCreateAntzChannelsTable()
{

	return 0;
}

int npCreateAntzTableMysql(struct dbConnect *connect, int tableType, void* dataRef)
{

	// These will return success or error, error checking, debug ll
	switch(tableType)
	{
	case NODE_STATE_TABLE:
		npCreateAntzNodeStateTable(connect, dataRef);
		break;
	case TAGS_TABLE:
		npCreateAntzTagsTable(connect, dataRef);
		break;
	case CHMAP_TABLE:
		npCreateAntzChMapTable(connect, dataRef);
		break;
	case CHANNELS_TABLE:
		npCreateAntzChannelsTable();
		break;	
	}

	return 0;
}

int npAntzTableMysql(struct dbConnect *connect, int command, int tableType, void* dataRef)
{
	
	// Error checking, debug ll
	switch(command)
	{
	case Create:
		npCreateAntzTableMysql(connect, tableType, dataRef);
		break;
	case Insert:
		npInsertAntzTableMysql(connect, tableType, dataRef);
		break;
	case Select:
		npSelectAntzTableMysql(connect, tableType, dataRef);
		break;
	case Drop:
		npDropAntzTableMysql(connect, tableType, dataRef);
		break;
	}

	return 0;
}

int npAntzMysql(struct dbConnect *connect, int objectType, int tableType, int command, void* dataRef)
{
	
	// Error checking, debug ll
	switch(objectType)
	{
	case DATABASE:
		npAntzDatabaseMysql(connect, command, dataRef);
		break;
	case TABLE:
		npAntzTableMysql(connect, command, tableType, dataRef);
		break;
	}

	return 0;
}

enum {
	ANTZ_DATABASE = 0
};

int npAntzDatabaseMysql(struct dbConnect *connect, int command, void* dataRef)
{
	
	// Error checking, debug ll
	switch(command)
	{
	case Create:
		npCreateAntzDatabaseMysql(connect, dataRef);
		break;
	case Use:
		npUseAntzDatabaseMysql(connect, dataRef);
		break;
	case Drop:
		//npDropAntzDatabaseMysql(connect) // Create this function, debug ll
		break;
	}
	
	return 0;
}

int npLoadAntzStateFromDatabase(struct dbConnect *connect, void* dataRef)
{	
	
	// Error checking, debug ll
//	npAntzMysql(connect, DATABASE, NULL, Create, dataRef); 

	npAntzMysql(connect, DATABASE, NULL, Use, dataRef); // I could eliminate this warning with a little restructureing, debug ll
	
	npAntzMysql(connect, TABLE, NODE_STATE_TABLE, Select, dataRef);
	
	npAntzMysql(connect, TABLE, TAGS_TABLE, Select, dataRef);
	return 0;	
}

int npSaveAntzStateToDatabase(struct dbConnect *connect, void* dataRef)
{
	// Error checking, debug ll
	
	npAntzMysql(connect, DATABASE, NULL, Create, dataRef); // For creating mysql tables which mirror the csv structure // I could eliminate this warning with a little restructureing, debug ll
	npAntzMysql(connect, DATABASE, NULL, Use, dataRef); // I could eliminate this warning with a little restructureing, debug ll

	npAntzMysql(connect, TABLE, TAGS_TABLE, Create, dataRef);
	npAntzMysql(connect, TABLE, TAGS_TABLE, Insert, dataRef);

	npAntzMysql(connect, TABLE, NODE_STATE_TABLE, Create, dataRef);
	npAntzMysql(connect, TABLE, NODE_STATE_TABLE, Insert, dataRef);

	npAntzMysql(connect, TABLE, CHMAP_TABLE, Create, dataRef);
	npAntzMysql(connect, TABLE, CHMAP_TABLE, Insert, dataRef); // It seems like I have unnecessary redundancy here, debug ll

//	npAntzMysql(connect, TABLE, CHANNELS_TABLE, Create, dataRef);
//	npAntzMysql(connect, TABLE, CHANNELS_TABLE, Insert, dataRef);
	
	return 0;
}

int npSetMysqlQueryConditionals(struct dbConnect *connect, struct mysqlObject *query, int conditionalType, char* conditional, void* dataRef)
{
	switch(conditionalType)
	{
		case WHERE:
			strcpy(((struct mysqlQuery*)query->myobject)->conditionals, conditional);
			((struct mysqlQuery*)query->myobject)->conditionalType = conditionalType;
			break;
	}
	
	return 0;
}

char*** npGetRootNodesIdsFromMysql(struct dbConnect *connect, char*** buffer, int interval, void* dataRef) // Add buffer
{
	char *conditional = malloc(200);
	struct mysqlObject *query = npInitMysqlQuery();
	struct mysqlObject *table = npNewInitMySQLTable("nodeStateTable");

	
	sprintf(conditional, "%s", "parent_id = 0");
	
	npUseAntzDatabaseMysql(connect, dataRef);
	npSetMysqlQueryObject(connect, query, table, dataRef);
	if(npSetMysqlQueryCommand(connect, query, mysqlSELECT, dataRef) == 0)
	{
		npSetMysqlQueryFields(connect, query, "id");
		npSetMysqlQueryConditionals(connect, query, WHERE, conditional, dataRef);
		npGenMysqlQuery(query, dataRef);
		npNewExecuteMysqlQuery(connect, query, dataRef);
		buffer = npGetResultMysql(connect, ((struct mysqlQuery*)query->myobject)->result[0], buffer, dataRef);
		printf("\nngrnrifm : %s", buffer[0][0]);

		//npLoadMysqlResultsIntoAntz(query, 1, dataRef); //enum??
	}
	//npFreeMysqlQuery(nodeSelectQuery);
	
	free(conditional);
	npFreeMysqlObject(connect, query, dataRef);
	npFreeMysqlObject(connect, table, dataRef);
	return buffer;
}

int npGetRootNodesFromMysql(struct dbConnect *connect, void* dataRef)
{
	char *conditional = malloc(200);
	struct mysqlObject *query = npInitMysqlQuery();
	struct mysqlObject *table = npNewInitMySQLTable("nodeStateTable");
	
		//sprintf(conditional, "%s", "branch_level = 0");
	sprintf(conditional, "%s", "parent_id = 0");
	//	sprintf(conditional, "(record_id >= %d && record_id <= %d) && (branch_level = 1 || branch_level = 0)", from, to);
	
	//	sprintf(conditional, "%s", "record_id % 2 = 0");
	npUseAntzDatabaseMysql(connect, dataRef);
	npSetMysqlQueryObject(connect, query, table, dataRef);
	if(npSetMysqlQueryCommand(connect, query, mysqlSELECT, dataRef) == 0)
	{
		npSetMysqlQueryConditionals(connect, query, WHERE, conditional, dataRef);
		npGenMysqlQuery(query, dataRef);
		npNewExecuteMysqlQuery(connect, query, dataRef);
		npLoadMysqlResultsIntoAntz(connect, query, 1, dataRef); //enum??
	}
	//npFreeMysqlQuery(nodeSelectQuery);
	
	free(conditional);
	npFreeMysqlObject(connect, query, dataRef);
	npFreeMysqlObject(connect, table, dataRef);
	return 0;
}

int npGetNodeRangeFromMysql(struct dbConnect *connect, char* field, char* from, char* to, void* dataRef) 
{
	char *conditional = malloc(200);
	struct mysqlObject *query = npInitMysqlQuery();
	struct mysqlObject *table = npNewInitMySQLTable("nodeStateTable");
	
	sprintf(conditional, "%s >= %s && %s <= %s", field, from, field, to);
//	sprintf(conditional, "%s", "record_id = 22564");

//	sprintf(conditional, "(record_id >= %d && record_id <= %d) && (branch_level = 1 || branch_level = 0)", from, to);
	
//	sprintf(conditional, "%s", "record_id % 2 = 0");
	npUseAntzDatabaseMysql(connect, dataRef);
	npSetMysqlQueryObject(connect, query, table, dataRef);
	if(npSetMysqlQueryCommand(connect, query, mysqlSELECT, dataRef) == 0)
	{
		npSetMysqlQueryConditionals(connect, query, WHERE, conditional, dataRef);
		npGenMysqlQuery(connect, query);
		npNewExecuteMysqlQuery(connect, query, dataRef);
		npLoadMysqlResultsIntoAntz(connect, query, 1, dataRef); //enum??
	}
	//npFreeMysqlQuery(nodeSelectQuery);
	free(conditional);
	npFreeMysqlObject(connect, query, dataRef);
	npFreeMysqlObject(connect, table, dataRef);
	return 0;
}
	

int npGetNodesWithMysql(struct dbConnect *connect, int fieldType, int fieldValue, void* dataRef) // Might want to make fieldValue a void*
{
	switch(fieldType)
	{
		
	}
	
	return 0;
}

char** npRetrieveChMapValues(char** buffer, int *numberOfChunks, void* dataRef)
{
	pData data = (pData) dataRef;		
	int i = 0;
	int count = 0;
	char *valsCopy = malloc(100 * data->io.ch.xrefIndex);
	data->io.write[kNPnode] = valsCopy;

	
	printf("\nNumber of channelMapValues :: %d", data->io.ch.xrefIndex);


	if(data->io.ch.xrefIndex == 0)
		return NULL;
	
	printf("\ndata->io.ch.xrefIndex :: %d", data->io.ch.xrefIndex);
	for (i = 0; i < data->io.ch.xrefIndex; i++)
	{
		count += sprintf((valsCopy + count), "%d,%d,%d,\"%s\",%d,%d,%d\n", i+1, data->io.ch.xref[i].channelNumber, 
						 data->io.ch.xref[i].trackOffset, data->io.ch.xref[i].attributeName, 0, 0, i+1);
	}
	
	printf("\nChannel Map :: -----\n%s\n-----", valsCopy);
	
	buffer = npNewChunkAllValues(valsCopy, numberOfChunks, dataRef);
	
	return buffer;
}

int npSampleNodeStateTable(struct dbConnect *connect, int mod, void* dataRef)
{
	char*** buffer = NULL;
	
	buffer = npGetRootNodesIdsFromMysql(connect, buffer, 2, dataRef); // npGetRootNodesIDs
	buffer = npNewFilterIDbyInterval(connect, buffer, mod, dataRef);

	npLoadRootNodes(connect, buffer, dataRef);
	
	npGetChildren(connect, buffer, dataRef);
	//npLoadChildren(buffer, dataRef);
	
	// Free buffer, debug ll
	// npFreeBuffer(void* buffer, int dimensions) ???, debug ll
	
	return 0;
}

char*** npGetChildNodeFromRecordIdMysql(struct dbConnect *connect, char* recordID, void* dataRef) // npGetChildNodesFromRecordIdMysql, change name
{
	char*** buffer = NULL;
	char condition[200] = {'\0'};
	struct mysqlObject *query = npInitMysqlQuery();
	struct mysqlObject *table = npNewInitMySQLTable("nodeStateTable");
	
	sprintf(condition, "record_id = %s", recordID);
	npSetMysqlQueryObject(connect, query, table, dataRef);
	printf("\nQuery focus : %d", ((struct mysqlQuery*)query->myobject)->focus->type);
	if(npSetMysqlQueryCommand(connect, query, mysqlSELECT, dataRef) == 0)
	{
		npSetMysqlQueryFields(connect, query, "child_id");
		npSetMysqlQueryConditionals(connect, query, WHERE, condition, dataRef);
		npGenMysqlQuery(query, dataRef);
		npNewExecuteMysqlQuery(connect, query, dataRef);
		buffer = npGetResultMysql(connect, ((struct mysqlQuery*)query->myobject)->result[0], buffer, dataRef);
		//npLoadMysqlResultsIntoAntz(query, 1, dataRef);
		//npLoadMysqlResultsIntoAntz(tagsSelectQuery, 0, dataRef); //enum??
	}
	//npFreeMysqlQuery(tagsSelectQuery);
	
	npFreeMysqlObject(connect, query, dataRef);
	npFreeMysqlObject(connect, table, dataRef);
	return buffer;
}

// This could be done more efficiently, select * from nodeStateTable where id IN (x1, x2, x3, x4, x5)
int npLoadRootNodes(struct dbConnect *connect, char*** buffer, void* dataRef)
{
	char *conditional = malloc(100000); // Make dynamic
	struct mysqlObject *query = npInitMysqlQuery();
	struct mysqlObject *table = npNewInitMySQLTable("nodeStateTable");
	int i = 0;
	int count = 0;

	
	//count += sprintf(conditional+count, "%s", "(");
	count += sprintf(conditional+count, "%s", "id IN (");
	while(buffer[i][0][0] != '\0')
	{
		count += sprintf(conditional+count, "%s,", buffer[i][0]);
		i++;
	}

	count--;
	conditional[count] = '\0';
	count += sprintf(conditional+count, "%s", ")");
	
	//printf("\nfound it: %s : %zu\n", conditional, strlen(conditional));
	
	npUseAntzDatabaseMysql(connect, dataRef);
	npSetMysqlQueryObject(connect, query, table, dataRef);
	if(npSetMysqlQueryCommand(connect, query, mysqlSELECT, dataRef) == 0)
	{
	//	npSetMysqlQueryFields(query, "id");
		npSetMysqlQueryConditionals(connect, query, WHERE, conditional, dataRef);
		npGenMysqlQuery(query, dataRef);
		npNewExecuteMysqlQuery(connect, query, dataRef);
		npLoadMysqlResultsIntoAntz(connect, query, 1, dataRef);
	}
		
	free(conditional);
	npFreeMysqlObject(connect, query, dataRef);
	npFreeMysqlObject(connect, table, dataRef);
	
	return 0;
}

int npGetChildren(struct dbConnect *connect, char*** buffer, void* dataRef)
{
	char *conditional = malloc(100000);
	struct mysqlObject *query = npInitMysqlQuery();
	struct mysqlObject *table = npNewInitMySQLTable("nodeStateTable");
	int i = 0;
	int count = 0;
	
	count += sprintf(conditional+count, "%s", "(");
	while(buffer[i+1][0][0] != '\0')
	{
		count += sprintf(conditional+count, "(%s >= %s && %s <= %s) || ", "id", buffer[i][0], "id", buffer[i+1][0]);
		printf("\nhave not found it : %s", buffer[i][0]);
		//npGetNodeRangeFromMysql(connect, "id", buffer[i][0], buffer[i+1][0], dataRef);
		i++;
	}
	
	count -= 3;
	conditional[count] = '\0';
	count += sprintf(conditional+count, ") %s", " && parent_id != 0");
	
	printf("\nfound it: %s : %zu\n", conditional, strlen(conditional));
	
	npUseAntzDatabaseMysql(connect, dataRef);
	npSetMysqlQueryObject(connect, query, table, dataRef);
	if(npSetMysqlQueryCommand(connect, query, mysqlSELECT, dataRef) == 0)
	{
		//	npSetMysqlQueryFields(query, "id");
		npSetMysqlQueryConditionals(connect, query, WHERE, conditional, dataRef);
		npGenMysqlQuery(query, dataRef);
		npNewExecuteMysqlQuery(connect, query, dataRef);
		npLoadMysqlResultsIntoAntz(connect, query, 1, dataRef);
	}
	
	free(conditional);
	npFreeMysqlObject(connect, query, dataRef);
	npFreeMysqlObject(connect, table, dataRef);
	
	return 0;
}

char*** npNewFilterIDbyInterval(struct dbConnect *connect, char ***buffer, int interval, void* dataRef)
{
	int i = 0, x = 0, filteredIndex = 0;
	char ***filteredIDs;

	while(buffer[i+1][0][0])
		i++;

	filteredIDs = malloc(sizeof(char*) * i);

	for(x = 0; x < i; x++)
	{
		if(x % interval == 0)
		{
			filteredIDs[filteredIndex] = malloc(sizeof(char*) * 1);
			filteredIDs[filteredIndex][0] = malloc(sizeof(char) * 30);
			strcpy(filteredIDs[filteredIndex][0], buffer[x][0]);
			printf("\nFilteredID %d : %s\n", filteredIndex, filteredIDs[filteredIndex][0]);
			filteredIndex++;
		}
		free(buffer[x][0]);
		free(buffer[x]);
	}

	filteredIDs[filteredIndex] = malloc(sizeof(char*) * 1);
	filteredIDs[filteredIndex][0] = malloc(sizeof(char) * 30);
	filteredIDs[filteredIndex][0][0] = '\0';

//	free(buffer);
	return filteredIDs;
}

char*** npFilterIDbyInterval(struct dbConnect *connect, char ***buffer, int interval, void* dataRef)
{
	int i = 0;
	int x = 0;
	int rows = 0;
	char ***filteredIDs;
	
	// Null terminate three dimensional array

	
	while ( buffer[i+1][0][0] != '\0')
	{
		i++;
	}
	
	rows = i;
	printf("Rows %d", rows);
	filteredIDs = malloc(sizeof(char*) * (rows));
	
//	for(i = 0; i < rows; i++)
//		filteredIDs[i] = malloc(sizeof(char*) * i); // Might be able to cut this apart by interval

	for(i = 0; i <= rows; i++)
		filteredIDs[i] = malloc(sizeof(char*) * 1); // Might be able to cut this apart by interval

/*
	i++;
	filteredIDs[i] = malloc(sizeof(char*) * 1);
	filteredIDs[i][0] = malloc(sizeof(char) * 2);
	filteredIDs[i][0][0] = '\0';
*/
	filteredIDs[i][0] = malloc(sizeof(char) * 2);
	filteredIDs[i][0][0] = '\0';

	for(i = 0; i < rows; i++)
	{
		if(i % interval == 0)
		{
			filteredIDs[x][0] = malloc(sizeof(char) * 30);
			printf("\nInterval %d : %s", i, buffer[i][0]);
			strcpy(filteredIDs[x][0], buffer[i][0]);
			//strcpy(buffer[i][0], buffer[i][0]);
			x++;
		}
		
	}
	
	printf("\nX is %d", x);
	for(i = 0; i < x; i++)
	{
		printf("\nFiltered ID %d : %s", i, filteredIDs[i][0]);
	}
	
	filteredIDs[x][0] = malloc(sizeof(char) * 30);
	filteredIDs[x][0][0] = '\0';
	
	// Completely free buffer
	for(i = 0; i < rows; i++)
	{
		free(buffer[i][0]);
		free(buffer[i]);
	}
	free(buffer);

	//printf("npFilterIDbyInterval : found null terminator %d", i+1);

	return filteredIDs;
}

int npLoadAntzTagsTable(struct dbConnect *connect, void* dataRef)
{
	npAntzMysql(connect, TABLE, TAGS_TABLE, Select, dataRef);
	return 0;
}

void npOldTest(void *dataRef)
{
	pData data = (pData) dataRef;
	int i = 0;
	int x = 0;
	char*** buffer = NULL;
	char ***childInfo = NULL;
	struct dbConnect *connect = malloc(sizeof(struct dbConnect));
	
	npInitMySQL(connect, "127.0.0.1", "root", "admin", "test");
	npOpenMySQL(connect, data);
	
	npUseAntzDatabaseMysql(connect, dataRef);
	//	npLoadAntzStateFromDatabase(connect, dataRef);
	//	npSaveAntzStateToDatabase(connect, dataRef);
	//	npGetNodeRangeFromMysql(connect, "record_id", 5000, 6000, dataRef);
	//	npGetNodeRangeFromMysql(connect, "translate_x", 170, 180, dataRef);
	
	// The sample function grabs every nth root node and its children, debug ll
	//npSampleNodeStateTable(connect, 5, dataRef);
	npLoadAntzTagsTable(connect, dataRef);
	
	//	npGetRootNodesFromMysql(connect, dataRef);
	//	pause();
	
	/*
	 buffer = npGetRootNodesRecordIdsFromMysql(connect, buffer, 2, dataRef); // npGetRootNodesIDs
	 buffer = npFilterIDbyInterval(buffer, 80);
	 printf("asdf %s", buffer[5][0]);
	 npGetChildren(connect, buffer, dataRef);
	 npGetChildren2(connect, buffer, dataRef);
	 */
	
	
	/*
	 printf("\n\n%p", buffer);
	 printf("\nngrnrifm : %s", buffer[0][1]);
	 
	 for(i = 0; i < 10; i++)
	 {
	 while(buffer[x][0][0] != '\0')
	 {
	 printf("\nbuffer %d %d : %s\n", i,x,childInfo[x][0]);
	 x++;
	 }
	 x = 0;
	 free(childInfo);
	 }
	 */
	
	//	npGetNodesWithMysql(struct dbConnect, int fieldType, int fieldValue)
	//	npAntzMysql(connect, TABLE, TAGS_TABLE, Select, dataRef);
	
	
	//	npSaveAntzStateToDatabase(connect, dataRef);
	
	/*
	 npAntzMysql(connect, TABLE, TAGS_TABLE, Create, dataRef);
	 npAntzMysql(connect, TABLE, NODE_STATE_TABLE, Create, dataRef);
	 npAntzMysql(connect, TABLE, TAGS_TABLE, Insert, dataRef);
	 npAntzMysql(connect, TABLE, NODE_STATE_TABLE, Insert, dataRef);
	 */
	
	
	
	
	/*
	 npCreateAntzDatabaseMysql(connect);
	 npUseAntzDatabaseMysql(connect);
	 
	 npAntzTableMysql(connect, Create, TAGS_TABLE, dataRef);
	 npAntzTableMysql(connect, Create, NODE_STATE_TABLE, dataRef);
	 npAntzTableMysql(connect, Insert, TAGS_TABLE, dataRef);
	 npAntzTableMysql(connect, Insert, NODE_STATE_TABLE, dataRef);
	 npAntzTableMysql(connect, Select, TAGS_TABLE, dataRef);
	 npAntzTableMysql(connect, Select, NODE_STATE_TABLE, dataRef);
	 npAntzTableMysql(connect, Drop, TAGS_TABLE, dataRef);
	 npAntzTableMysql(connect, Drop, NODE_STATE_TABLE, dataRef);
	 */
}

int npAttachMySQLtoDataRef(struct dbConnect *connect, void* dataRef)
{
	pData data = (pData) dataRef;
	data->io.connect = connect;
	return 0;
}

void npNewTest(void *dataRef)
{
	/*
	pData data = (pData) dataRef;
	struct dbConnect *connect = malloc(sizeof(struct dbConnect));

	npInitMySQL(connect, "127.0.0.1", "root", "admin", "test");
	npOpenMySQL(connect, data);
	npAttachMySQLtoDataRef(connect, dataRef);
	*/
	
//	npUseAntzDatabaseMysql(connect, dataRef);
//	npSaveAntzStateToDatabase();
//	npLoadAntzStateFromDatabase(connect, dataRef);
	
//	npSampleNodeStateTable(5);

	// The sample function grabs every nth root node and its children, debug ll
//	npSampleNodeStateTable(connect, 5, dataRef);
//	npLoadAntzTagsTable();

}


struct mysqlObject* npInitMySQLDatabase(char* name)
{
	struct mysqlObject *object = malloc(sizeof(struct mysqlObject));
	struct mysqlDatabase *tmpDatabase;
	
	object->type = DATABASE;
	object->myobject = malloc(sizeof(struct mysqlDatabase));
	tmpDatabase = (struct mysqlDatabase*)object->myobject;
	tmpDatabase->name = malloc(10); //Dynamic
	
	strcpy(tmpDatabase->name, name);
	printf("\nname = %s", ((struct mysqlDatabase*)object->myobject)->name);
	printf("\nname = %s", tmpDatabase->name);
	
	return object;
}

struct mysqlObject* npNewInitMySQLTable(char *name)
{
	struct mysqlObject *object = malloc(sizeof(struct mysqlObject));
	struct mysqlTable *tmpTable;
	object->type = TABLE;
	object->myobject = malloc(sizeof(struct mysqlTable));
	tmpTable = (struct mysqlTable*)object->myobject;
	
	//-----
	tmpTable->name = malloc(strlen(name)*sizeof(char));
	strcpy(tmpTable->name, name);
	tmpTable->fields = malloc(3000);
	tmpTable->fieldsSize = 3001;
	tmpTable->fields[0] = '\0';
	
	printf("\nfieldSize = %d", ((struct mysqlTable*)object->myobject)->fieldsSize);
	printf("\nfieldSize = %d", tmpTable->fieldsSize);
	//-----

	return object;
}

struct mysqlObject* npInitMysqlQuery()
{
	struct mysqlObject *object = malloc(sizeof(struct mysqlObject));
	struct mysqlQuery *tmpQuery;
	object->type = QUERY;
	object->myobject = malloc(sizeof(struct mysqlQuery));
	tmpQuery = (struct mysqlQuery*)object->myobject;
	((struct mysqlQuery*)object->myobject)->numberOfQueries = 1;
	((struct mysqlQuery*)object->myobject)->numOfResults = 0;
	((struct mysqlQuery*)object->myobject)->conditionalType = -1; // If set to -1, conditional is not set
	((struct mysqlQuery*)object->myobject)->values = NULL;
	((struct mysqlQuery*)object->myobject)->fields = NULL;
	//-----
/*
	tmpTable->name = malloc(strlen(name)*sizeof(char));
	strcpy(tmpTable->name, name);
	tmpTable->fields = malloc(3000);
	tmpTable->fieldsSize = 3001;
	tmpTable->fields[0] = '\0';
	
	printf("\nfieldSize = %d", ((struct mysqlTable*)object->myobject)->fieldsSize);
	printf("\nfieldSize = %d", tmpTable->fieldsSize);
*/
	//-----
	
	return object;
}

int npGenMysqlFields(char* fields, int count, int type, void* dataRef)
{
	int i = 0;
	int itemCount = 0;
	
	NPmapTypePtr map = NULL;
	
	pData data = (pData) dataRef;
	
	map = data->map.typeMapNode;		//debug, zz
	

		if (fields == NULL || count < 5)
			return 0;
		
		//set map and item count based on type, could be passed in, debug zz
		switch (type)
		{
			case kNPnode: 
				map = data->map.typeMapNode;
				itemCount = kNPnodeItemCount;
				break;
			case kNPcamera :
				map = data->map.typeMapCamera;
				itemCount = kNPcameraItemCount;
				break;
			case kNPgrid :
				map = data->map.typeMapGrid;
				itemCount = kNPgridItemCount;
				break;
			case kNPpin :
				map = data->map.typeMapPin;
				itemCount = kNPpinItemCount;
				break;
			case kNPtag :
				map = data->map.typeMapTag;
				itemCount = kNPtagItemCount;
				break;
			case kNPchMap :
				map = data->map.typeMapChMap;
				itemCount = kNPChMapItemCount;
				break;
			default : 
				printf ("err 4381 - unknown type, cant write header field names");
				return 0;	
				break;
		}
		
		//	// This seems a little kludgy, debug ll
		//	((struct mysqlTable*)object->myobject)->fields = malloc(itemCount * 70); // sizeof char* or char ???, debug ll
		//	printf("\nbuffer pntr :: %p\n", ((struct mysqlTable*)object->myobject)->fields);
		
		count = 0; 
		for (i=0; i < itemCount; i++)
		{
			// insert a comma for CSV and JSON files
			//first field name to lead without a comma one without comma
			if (i > 0)
				count += sprintf ((fields + count), ",");
			
			switch (map[i].type)
			{
				case 0 :				// same as default
					count += sprintf ((fields + count), "%s INT(10)", map[i].name );
					break;
				case kNPcharArray :
					count += sprintf ((fields + count), "%s VARCHAR(250)", map[i].name ); // might want to have some sort of setup so that size is variable, debug ll
					break;
				case kNPfloatXYZ : 
					count += sprintf ((fields + count), "%s_x FLOAT(10),%s_y FLOAT(10),%s_z FLOAT(10)",
									  map[i].name, map[i].name, map[i].name );
					break;
				case kNPintXYZ : 
					count += sprintf ((fields + count), "%s_x INT(10),%s_y INT(10),%s_z INT(10)",
									  map[i].name, map[i].name, map[i].name );
					break;
				case kNPboolXYZ : 
					count += sprintf ((fields + count), "%s_x INT(10),%s_y INT(10),%s_z INT(10)",
									  map[i].name, map[i].name, map[i].name );	// Don't know what to do with BOOLS in MySQL, just calling it INT(10) for now, debug ll....should be TINYINT(1)
					break;
				case kNPfloatXYZA : 
					count += sprintf ((fields + count), "%s_x FLOAT(10),%s_y FLOAT(10),%s_z FLOAT(10),%s_a FLOAT(10)",
									  map[i].name, map[i].name, map[i].name, map[i].name );
				case kNPfloatXYZS : 
					count += sprintf ((fields + count), "%s_x FLOAT(10),%s_y FLOAT(10),%s_z FLOAT(10),%s_s FLOAT(10)",
									  map[i].name, map[i].name, map[i].name, map[i].name );
					break;
				case kNPboolXYZS : 
					count += sprintf ((fields + count), "%s_x INT(10),%s_y INT(10),%s_z INT(10),%s_s INT(10)",
									  map[i].name, map[i].name, map[i].name, map[i].name );
					break;
				case kNPubyteRGBA : 
					count += sprintf ((fields + count), "%s_r INT(10),%s_g INT(10),%s_b INT(10),%s_a INT(10)",
									  map[i].name, map[i].name, map[i].name, map[i].name );
					break;
				case kNPubyteRGB : 
					count += sprintf ((fields + count), "%s_r INT(10),%s_g INT(10),%s_b INT(10)",
									  map[i].name, map[i].name, map[i].name );
					break;
				case kNPfloatRGBA : 
					count += sprintf ((fields + count), "%s_r INT(10),%s_g INT(10),%s_b INT(10),%s_a INT(10)",
									  map[i].name, map[i].name, map[i].name, map[i].name );
					break;
				case kNPfloatRGB : 
					count += sprintf ((fields + count), "%s_r INT(10),%s_g INT(10),%s_b INT(10)", 
									  map[i].name, map[i].name, map[i].name );
					break;
					
				default :				// same as case 0
					count += sprintf ((fields + count), "%s INT(10)", map[i].name );
					break;
			}
		}

	
	//	count += sprintf ((((struct mysqlTable*)object->myobject)->fields + count), "\n");
	
	return count;
	
}


//------------------------------------------------------------------------------
void npInitMySQL (struct dbConnect *connection, char* host, char* user, char* pass, char* db)
{	
//	pData data = (pData) dataRef;
	connection->host = malloc(strlen(host)+1);
	strcpy(connection->host, host);

	connection->user = malloc(strlen(user)+1);
	strcpy(connection->user, user);
	//strcpy(connection->user, "root");
	printf("\ndatabase username is %s", connection->user);
	printf("\ndatabase password is %s", pass);

	connection->pass = (char*)malloc(strlen(pass)+1);
	printf("\nBefore 2");
	strcpy(connection->pass, pass);

	printf("\nAfter");
//	connection->db = malloc(strlen(db)+1);
//	strcpy(connection->db, db);
	connection->db = NULL;
	
	connection->q = malloc(sizeof(struct query));
	printf("\nReturning");
	return;
}

void npOpenMySQL (struct dbConnect *connection, void* dataRef)
{
//	pData data = (pData) dataRef;
//	data->io.npdb.conn = mysql_init(NULL);
	connection->conn = mysql_init(NULL);
	connection->conn = mysql_real_connect(connection->conn, connection->host, "root", connection->pass, NULL, 0, NULL, 0);	
//	data->io.npdb.conn = mysql_real_connect(data->io.npdb.conn, data->io.npdb.host, data->io.npdb.user, data->io.npdb.pass, NULL, 0, NULL, 0);

/*
	if (data->io.npdb.conn == NULL)
	{
		printf("\nUnable to establish connection\n");
	}
	else {
		printf("\nConnection Established\n");
	}
*/
	if (connection->conn == NULL)
		printf("\nUnable to establish connection\n");
	else
		printf("\nConnection Established\n");

	return;
}

//------------------------------------------------------------------------------
void npCloseMySQL (struct dbConnect *connect, void* dataRef)		//clean-up
{
	return;
}

//------------------------------------------------------------------------------
void npUpdateMySQL (struct dbConnect *connect, void* dataRef)							//add to ctrl loop, debug zz
{
	return;
}

void npSetMysqlQueryObject(struct dbConnect *connect, struct mysqlObject* query, struct mysqlObject *dbOrTable, void* dataRef)
{
	((struct mysqlQuery*)query->myobject)->focus = dbOrTable;
	printf("\nnpSetMysqlQueryObject type : %d", ((struct mysqlQuery*)query->myobject)->focus->type);
}

int npSetMysqlQueryCommand(struct dbConnect *connect, struct mysqlObject *query, int command, void* dataRef)
{

	// Return error if command and object type doesn't work out, Error Checking debug ll

	switch (((struct mysqlQuery*)query->myobject)->focus->type)
	{
		case DATABASE:
			switch(command)
			{
				case mysqlSELECT:
					printf("\nCannot perform SELECT query on a database");
					return -1;
					break;
				case mysqlINSERT:
					printf("\nCannot perform INSERT query on a database");
					return -1;
					break;
				case mysqlDELETE:
					printf("\nCannot perform DELETE query on a database");
					return -1;
					break;
				case mysqlUPDATE:
					printf("\nCannot perform UPDATE query on a database");
					return -1;
					break;
				case mysqlALTER:
					printf("\nCannot perform ALTER query on a database");
					return -1;
					break;
			}
			break;
		case TABLE:
			switch(command)
			{
			case mysqlUSE:
				printf("\nCannot perform USE query on a table");
				return -1;
				break;
			}
			break;
		case QUERY:
			// Add support for nested queries, debug ll
			break;
	}
	((struct mysqlQuery*)query->myobject)->command = command;
	return 0;
}

//Think all return values for error checking
void npNewExecuteMysqlQuery(struct dbConnect *connect, struct mysqlObject *object, void* dataRef)
{
//	MYSQL_RES *result;
	void *result;
	int i = 0;
	int *ErrResult = 0;
	//int ErrResult = 0;

	printf("npNewExecuteMysqlQuery :: %d", ((struct mysqlQuery*)object->myobject)->numberOfQueries);
	//pause();
	printf("\nQuery : %s FTL", ((struct mysqlQuery*)object->myobject)->query[0]);

	for(i = 0; i < ((struct mysqlQuery*)object->myobject)->numberOfQueries; i++)
	{
		printf("\nQuery Number : %d", i);

		((struct mysqlQuery*)object->myobject)->result[i] = npExecuteQueryBufferMySQL(connect, ((struct mysqlQuery*)object->myobject)->query[i], connect, ErrResult, dataRef);
//		((struct mysqlQuery*)object->myobject)->result[i] = npExecuteQueryBufferMySQL(connect, ((struct mysqlQuery*)object->myobject)->query[i], connect, ErrResult, dataRef);
//		if(((struct mysqlQuery*)object->myobject)->result[i] == NULL)
//			printf("\nresult is NULL");
		
		if(ErrResult != 0)
		{
			printf("\nResult of query : %i", ErrResult);
			printf("\n%d", mysql_errno(connect->conn));
			printf("The offending query is %s", ((struct mysqlQuery*)object->myobject)->query[i]);
		}

		if(((struct mysqlQuery*)object->myobject)->result[i] != NULL)
		{
			((struct mysqlQuery*)object->myobject)->numOfResults++;
		}

	}
}

void* npExecuteQueryBufferMySQL(struct dbConnect *connect, char* buffer, struct dbConnect *connection, int *ErrResult, void* dataRef)
{
	MYSQL_RES *result;
	MYSQL *conn = connect->conn;
	int test = 0;
//	int ErrResult = 0;
	printf("\nExec : %s\n", buffer);

//	printf("\nErr Result of Query : %d\n", mysql_query(conn, buffer));
	ErrResult = mysql_query(connect->conn, buffer);
//	ErrResult = mysql_query(conn, buffer);
//	ErrResult = test;
//	mysql_query(connect->conn, buffer);

	printf("\nErrResult : %d\n", ErrResult);

	result = mysql_store_result(connect->conn);
	if(result == NULL)
	{
		printf("\nnpExecuteQueryBufferMysql NULL");
		return NULL;
	}

//	memset(buffer, '\0', strlen(buffer));

//	printf("\n----The Query is %s-----\n", buffer);
	
	return result;
}

void npLoadMysqlResultsIntoAntz(struct dbConnect *connect, struct mysqlObject *object, int tableType, void* dataRef)
{
	
	enum{
		TAGS = 0,
		NODES = 1,
		CHMAP = 2,
		CHANNELS = 3
	};
	
//	char *tagsValues;
//	int scanNumRet;
	printf("tableType : %d", tableType);
	switch (tableType)
	{
		case TAGS:
			//tagsValues = parseResult(((struct mysqlQuery*)object->myobject)->result[0], dataRef);
			//printf("\ntagsValues : %s\n", tagsValues);
				//   pause();
			//npLoadTagBuffer(tagsValues, 0, &scanNumRet, dataRef);
			npGetTextTagsFromMySQL(((struct mysqlQuery*)object->myobject)->result[0], dataRef);
			break;
		case NODES:
			npLoadMysqlNodeStateResultsIntoAntz(object, dataRef);
			break;
		case CHMAP:
			break;
		case CHANNELS:
			break;
	}
		
}

void npGetTextTagsFromMySQL(MYSQL_RES *result, void* dataRef)
{
	pData data = (pData) dataRef;
	pNPrecordTag recordTag = NULL;
//	int id = 0, record_id = 0, table_id = 0;
//	int count = 0;
//	char *buffer = malloc(50);
	MYSQL_ROW row;
	printf("\nnpGetTextTagsFromMySQL LL");
	
	while( row = mysql_fetch_row(result) )
	{
		recordTag = (pNPrecordTag) malloc(sizeof(NPrecordTag)); 
		if (recordTag == NULL)
		{
			printf ("err 4279 - malloc failed making an NPrecordTag\n");
			return;
		}
		npInitDataRecordTag(recordTag, data);
		
		//copy the id fields
		recordTag->id = atoi(row[0]);
		recordTag->recordID = atoi(row[1]);
		recordTag->tableID = atoi(row[2]);
		strcpy(recordTag->title, row[3]);
		strcpy(recordTag->desc, row[4]);		
		printf("\nTitle is %s", recordTag->title);
		printf("\nDescription is %s", recordTag->desc);
	//	pause();
		npAddRecordTag (recordTag, dataRef);
		
	}
	npSyncTags(dataRef);
	
	free(recordTag);

	/*
	if (count > kNPtitleSizeMax)
	{
		printf("err 4959 - record title exceeds kNPtitleSizeMax\n");
		//need to copy then advance buffer
		strncpy(recordTag->title, buffer, kNPtitleSizeMax);
		recordTag->title[kNPtitleSizeMax] = '\0';
		buffer = &buffer[count];
		count = kNPtitleSizeMax;
	}
	else
	{//copy the string and add a null character
		strncpy(recordTag->title, buffer, count);
		recordTag->title[count] = '\0';
		buffer = &buffer[count];
	}
	
	recordTag->titleSize = count;
	printf ("titleSize: %d  ", count);
	printf ("\nrecordTag->title = %s\n", recordTag->title);
	
	buffer++;
	//repeat process for desc
	//next character should be the beginning double quote of the title field
	if (buffer[0] != ',' && buffer[1] != '\"')		//upgrade to allow for a number or no title, debug zz
	{
		printf ("\n");	
		printf ("err 4912 - desc string missing double quote\n");
		
		//permits recordTag with title but no desc
		npAddRecordTag (recordTag, dataRef);
		return recordTag;
	}
	
	buffer++;
	buffer++;
	
	count = 0;
	//will need to convert to "" to '\"'								//debug zz
	while (buffer[count] != '\"' || buffer[count+1] == '\"' )
	{
		if (buffer[count] == '\"' && buffer[count+1] == '\"')
			count++;
		
		count++;
	}
	
	if (count > kNPdescSizeMax)
	{
		printf("err 4960 - record description exceeds kNPdescSizeMax\n");
		//need to copy then advance buffer
		strncpy(recordTag->desc, buffer, kNPdescSizeMax);
		recordTag->desc[kNPdescSizeMax] = '\0';
		buffer = &buffer[count];
		count = kNPdescSizeMax;
	}
	else
	{//copy the string and add a null character
		strncpy(recordTag->desc, buffer, count);
		recordTag->desc[count] = '\0';
		buffer = &buffer[count];
	}
	recordTag->descSize = count;
	printf ("descSize: %d\n", count);
	printf ("\nrecordTag->desc = %s\n", recordTag->desc);
	
	//add the recordTag to the recordTag list
	npAddRecordTag (recordTag, dataRef);
	
	return recordTag;	
	 */
}

void npLoadMysqlNodeStateResultsIntoAntz(struct mysqlObject *object, void* dataRef)
{	
	int i = 0;
	printf("\nnumOfResults : %d", ((struct mysqlQuery*)object->myobject)->numOfResults);
	for(i = 0; i < ((struct mysqlQuery*)object->myobject)->numOfResults; i++)
	{
		npLoadNodeStateResultIntoAntz(((struct mysqlQuery*)object->myobject)->result[i], dataRef);
	}
	
}

char*** npGetResultMysql(struct dbConnect *connect, MYSQL_RES *result, char*** buffer, void* dataRef)
{
	int numFields = 0;
	int numRows = 0;
	unsigned long *lengths;
	int i = 0;
	int x = 0;
	MYSQL_ROW row;

	numFields = mysql_num_fields(result);
	numRows = mysql_num_rows(result);
	buffer = malloc(sizeof(char*) * numRows+1);
	
	printf("\nNumber of rows : %d", numRows);
	for(i = 0; i < numRows; i++)
		buffer[i] = malloc(sizeof(char*) * numFields);
		
	// Null terminate three dimensional array
	buffer[i] = malloc(sizeof(char*) * 1);
	buffer[i][0] = malloc(sizeof(char) * 1);
	buffer[i][0][0] = '\0';
	
	for(i = 0; i < numRows; i++)
	{
		printf("\n");
		row = mysql_fetch_row(result);
		lengths = mysql_fetch_lengths(result);
		for(x = 0; x < numFields; x++)
		{
			buffer[i][x] = malloc((sizeof(char) * lengths[x]) + 150);
			strcpy(buffer[i][x], row[x]); // I might be able to use a memcpy or memmove here to get a speed increase, debug ll
			printf("%s", buffer[i][x]);
			if( x != (numFields - 1) )
				printf(",");
				
		}
	}
	
	if( (buffer[i][0][0]) == '\0' )
		printf("\nfound null terminator");
	
	// Remember to free the result
	mysql_free_result(result);
	
	return buffer;
}

void npLoadNodeStateResultIntoAntz(MYSQL_RES *result, void* dataRef)
{
	MYSQL_ROW row;
//	int numFields, i = 0;
	int x = 0;
	int count = 0;
//	buffer = malloc(100000);
	time_t start;
	time_t end;
	pNPnode node = NULL;
	pNPnode nodeParent = NULL;
	int id = 0, type = 0, branchLevel = 0, parentID = 0;
	// Node Data Start
	pData data = (pData) dataRef;

	
	// Node Data End
	
//	memset(buffer, '\0', 1500);
	//numFields = mysql_num_fields(result);
	
	// This could be optimized, strcat needs to traverse to the end of buffer everytime it runs,
	// With each successive loop it takes longer. A better solution would be to simply insert row[i] and ',' from the end each time,
	// thus incurring no performance penalty.
	//	printf("\nFIELDS: %s\n", result->fields); pause();
	start = time(NULL);
	while( row = mysql_fetch_row(result) )
	{
			//printf("\nNew Node", x);
		x++;
		id = atoi(row[0]);
		type = atoi(row[1]);
		branchLevel = atoi(row[5]);
		parentID = atoi(row[4]);
		node = npMapNodeAdd (id, type, branchLevel, parentID, kNPmapNodeCSV, dataRef); // kNPmapNodeCSVvTwo is format		
		

		node->selected = atoi(row[3]);
		node->childIndex = atoi(row[7]);
		
		node->chInputID		= atoi(row[9]);
		node->chOutputID	= atoi(row[10]);
		node->chLastUpdated	= atoi(row[11]);
		
		node->average		= atoi(row[12]);
		node->interval		= atoi(row[13]); // Samples???
		
		node->auxA.x		= atoi(row[14]);
		node->auxA.y		= atoi(row[15]);
		node->auxA.z		= atoi(row[16]);
		node->auxB.x		= atoi(row[17]);
		node->auxB.y		= atoi(row[18]);
		node->auxB.z		= atoi(row[19]);
		
		node->colorShift	= atoi(row[20]);
		
		node->rotateVec.x		= atof(row[21]);		//was rotate
		node->rotateVec.y		= atof(row[22]);
		node->rotateVec.z		= atof(row[23]);
		node->rotateVec.angle	= atof(row[24]);	
		
		node->scale.x		= atof(row[25]);
		node->scale.y		= atof(row[26]);
		node->scale.z		= atof(row[27]);
		
		node->translate.x	= atof(row[28]);
		node->translate.y	= atof(row[29]);
		node->translate.z	= atof(row[30]);
		
		node->tagOffset.x	= atof(row[31]);
		node->tagOffset.y	= atof(row[32]);
		node->tagOffset.z	= atof(row[33]);
		
		node->rotateRate.x	= atof(row[34]);
		node->rotateRate.y	= atof(row[35]);
		node->rotateRate.z	= atof(row[36]);
		
		node->rotate.x		= atof(row[37]);					//was rotateRad
		node->rotate.y		= atof(row[38]);
		node->rotate.z		= atof(row[39]);
		
		node->scaleRate.x	= atof(row[40]);
		node->scaleRate.y	= atof(row[41]);
		node->scaleRate.z	= atof(row[42]);
		
		node->translateRate.x = atof(row[43]);
		node->translateRate.y = atof(row[44]);
		node->translateRate.z = atof(row[45]);
		
		node->translateVec.x = atof(row[46]);
		node->translateVec.y = atof(row[47]);
		node->translateVec.z = atof(row[48]);
		
		node->shader		= atoi(row[49]);
		node->geometry		= atoi(row[50]);
		
		node->lineWidth		= atoi(row[51]);
		node->pointSize		= atoi(row[52]);
		node->ratio			= atoi(row[53]);
		
		node->colorIndex	= atoi(row[54]);
		
		node->color.r		= atoi(row[55]);
		node->color.g		= atoi(row[56]);
		node->color.b		= atoi(row[57]);
		node->color.a		= atoi(row[58]);
		
		node->colorFade		= atoi(row[59]);
		node->textureID		= atoi(row[60]);
		
		node->hide			= atoi(row[61]);
		node->freeze		= atoi(row[62]);
		
		//	node->center		= center;		//removed	
		
		node->topo			= atoi(row[63]);			//moved topo
		node->facet			= atoi(row[64]);		//added topo facet number
		
		node->autoZoom.x	= atoi(row[65]);	//moved down a slot
		node->autoZoom.y	= atoi(row[66]);
		node->autoZoom.z	= atoi(row[67]);
		
		//	node->scroll		= scroll;		//removed made space for facet, zz debug
		
		node->triggerHi.x	= atoi(row[68]);
		node->triggerHi.y	= atoi(row[69]);
		node->triggerHi.z	= atoi(row[70]);
		
		node->triggerLo.x	= atoi(row[71]);
		node->triggerLo.y	= atoi(row[72]);
		node->triggerLo.z	= atoi(row[73]);
		
		node->setHi.x		= atof(row[74]);
		node->setHi.y		= atof(row[75]);
		node->setHi.z		= atof(row[76]);
		
		node->setLo.x		= atof(row[77]);
		node->setLo.y		= atof(row[78]);
		node->setLo.z		= atof(row[79]);
		
		node->proximity.x	= atof(row[80]);
		node->proximity.y	= atof(row[81]);
		node->proximity.z	= atof(row[82]);
		
		node->proximityMode.x = atoi(row[83]);			//int cast supports 1st ver CSV
		node->proximityMode.y = atoi(row[84]);
		node->proximityMode.z = atoi(row[85]);
		
		node->segments.x	= atoi(row[86]);		//grid segments were stored in node->data,
		node->segments.y	= atoi(row[87]);		//was node->data->segments.x
		node->segments.z	= atoi(row[88]);		//now node->segments.x
		
		node->tagMode		= atoi(row[89]);
		node->formatID		= atoi(row[90]);
		node->tableID		= atoi(row[91]);
		node->recordID		= atoi(row[92]);
		//node->size		= lineSize;				// handled during node creation
		
		//support for first version CSV
	//	if (format == kNPmapNodeCSVvOne)
		//	npMapCSVvOne (node);
		
		//file compatability prior to 2012-04-22
		if (node->topo == 0 && node->type == kNodePin)
		{
			//printf ("topo = 0   id: %d\n", node->id);
			if (node->branchLevel == 0)
				node->topo = kNPtopoPin;	//set root topo to a pin
			else if (node->parent != NULL)  //orhpan child methods in npMapSort
			{
				nodeParent = node->parent;
				if ( nodeParent->topo == kNPtopoPin || nodeParent->topo == 0
					|| nodeParent->topo == kNPtopoTorus )
					node->topo = kNPtopoTorus;
				else
					node->topo = kNPtopoPin;
		}
			
			
			
	}
		
	}
	//buffer[count] = '\0';
	end = time(NULL);
//	printf("\nDifference TIME : %f\n", difftime(end, start));
//	printf("\nNumber of nodes added : %d", x);
//	printf("\nDONE PARSING : %s\n", buffer);// pause();
	//printf("\nnpParseResultMySQL :: \n%s\n", buffer);
	
	
	if(result != NULL)
	{
		mysql_free_result(result);
	}
	//	return buffer;
}

void npGenMysqlQuery(struct mysqlObject *object, void* dataRef)
{
	struct mysqlObject *focused = ((struct mysqlQuery*)object->myobject)->focus;
	printf("\nnpGenMysqlQuery type : %d", focused->type);
	
	switch (focused->type)
	{
		case DATABASE:
		//	sprintf(query, "%s DATABASE %s", );
			npGenMysqlDatabaseQuery(object);
			break;
		case TABLE:
			npGenMysqlTableQuery(object);
			break;
		case QUERY:
			
			printf("\nINVALID REQUEST\n");
			break;
	}
}



void npGenMysqlTableQuery(struct mysqlObject *object)
{
	// Check to make sure mysqlObject is of type query
	// Add options later for selecting database options like engines
	
	// Not sure if USE or SHOW belongs here
	int numOfQueries = 0;
	int i = 0;
	
	char buffer[500000] = {'\0'};
	// Check that focused actually points somewhere
	struct mysqlObject *focused = ((struct mysqlQuery*)object->myobject)->focus->myobject;
	
	
	printf("\nCOMMAND IS %d\n",((struct mysqlQuery*)object->myobject)->command);
	switch (((struct mysqlQuery*)object->myobject)->command)
	{
		case mysqlALTER:
			break;
			
		case mysqlCREATE:
			printf("\nTABLE NAME IS %s\n", ((struct mysqlTable*)focused)->name);
			printf("\nTABLE FIELDS ARE %s\n", ((struct mysqlQuery*)object->myobject)->fields);
			sprintf(((struct mysqlQuery*)object->myobject)->query[0], "CREATE TABLE %s (%s)", ((struct mysqlTable*)focused)->name, ((struct mysqlQuery*)object->myobject)->fields);
			printf("\nnpGenMysqlTableQuery : %s\n", ((struct mysqlQuery*)object->myobject)->query[0]);
			//	pause();
			break;
			
		case mysqlDROP:
			printf("\nTABLE NAME IS %s\n", ((struct mysqlDatabase*)focused)->name);
			sprintf(((struct mysqlQuery*)object->myobject)->query[0], "DROP TABLE %s", ((struct mysqlTable*)focused)->name);
			printf("\nnpGenMysqlTableQuery : %s\n", ((struct mysqlQuery*)object->myobject)->query[0]);
			break;
			
		case mysqlINSERT:
			printf("\nTABLE NAME IS %s\n", ((struct mysqlDatabase*)focused)->name);
			for(i = 0; i < ((struct mysqlQuery*)object->myobject)->numberOfQueries; i++)
			{
				sprintf(((struct mysqlQuery*)object->myobject)->query[i], "INSERT INTO %s VALUES %s", ((struct mysqlDatabase*)focused)->name, ((struct mysqlQuery*)object->myobject)->values[i]);
				//printf("\nnpGenMysqlTableQuery %d : %s\n",i, ((struct mysqlQuery*)object->myobject)->query[i]);
			}
			break;
			
		case mysqlSELECT:
			printf("\nTABLE NAME IS %s\n", ((struct mysqlDatabase*)focused)->name);
			
			switch (((struct mysqlQuery*)object->myobject)->conditionalType)
			{
				case WHERE:
					if(((struct mysqlQuery*)object->myobject)->fields == NULL)
					{
					sprintf(((struct mysqlQuery*)object->myobject)->query[0], "SELECT * FROM %s WHERE %s", ((struct mysqlDatabase*)focused)->name, ((struct mysqlQuery*)object->myobject)->conditionals);
					printf("WHERE : %s", ((struct mysqlQuery*)object->myobject)->query[0]);
					}
					else {
						sprintf(((struct mysqlQuery*)object->myobject)->query[0], "SELECT %s FROM %s WHERE %s",((struct mysqlQuery*)object->myobject)->fields ,((struct mysqlDatabase*)focused)->name, ((struct mysqlQuery*)object->myobject)->conditionals);
						printf("WHERE : %s", ((struct mysqlQuery*)object->myobject)->query[0]);
					}

					//pause();
					break;
					
				case ORDERBY:
					break;
					
				case -1:
					sprintf(((struct mysqlQuery*)object->myobject)->query[0], "SELECT * FROM %s", ((struct mysqlDatabase*)focused)->name);
					break;
				
			}
			
			break;
			
		case mysqlUSE:
			break;
			
		case mysqlSHOW:
			break;
			
	}
	
}

void npGenMysqlDatabaseQuery(struct mysqlObject *object)
{
	// Check to make sure mysqlObject is of type query
	// Add options later for selecting database options like engines
	
	// Not sure if USE or SHOW belongs here

	
	char buffer[50] = {'\0'};
	// Check that focused actually points somewhere
	struct mysqlObject *focused = ((struct mysqlQuery*)object->myobject)->focus->myobject;


	printf("\nCOMMAND IS %d\n",((struct mysqlQuery*)object->myobject)->command);
	switch (((struct mysqlQuery*)object->myobject)->command)
	{
		case mysqlALTER:
			break;
			
		case mysqlCREATE:
			printf("\nDATABASE NAME IS %s\n", ((struct mysqlDatabase*)focused)->name);
			sprintf(((struct mysqlQuery*)object->myobject)->query[0], "CREATE DATABASE %s", ((struct mysqlDatabase*)focused)->name);
			printf("\nnpGenMysqlDatabaseQuery : %s\n", ((struct mysqlQuery*)object->myobject)->query[0]);
		//	pause();
			break;
		
		case mysqlDROP:
			printf("\nDATABASE NAME IS %s\n", ((struct mysqlDatabase*)focused)->name);
			sprintf(((struct mysqlQuery*)object->myobject)->query[0], "DROP DATABASE %s", ((struct mysqlDatabase*)focused)->name);
			printf("\nnpGenMysqlDatabaseQuery : %s\n", ((struct mysqlQuery*)object->myobject)->query[0]);
			break;
			
		case mysqlUSE:
			printf("\nDATABASE NAME IS %s\n", ((struct mysqlDatabase*)focused)->name);
			sprintf(((struct mysqlQuery*)object->myobject)->query[0], "USE %s", ((struct mysqlDatabase*)focused)->name);			
			printf("\nnpGenMysqlDatabaseQuery : %s\n", ((struct mysqlQuery*)object->myobject)->query[0]);
			break;
			
		case mysqlSHOW:
			break;
			
	}
}

void npSetMysqlQueryFields(struct dbConnect *connect, struct mysqlObject *object, char *fields)
{
	//Check to see if object type supports fields
	((struct mysqlQuery*)object->myobject)->fields = fields;
	
}

void npSetMysqlQueryValues(struct dbConnect *connect, struct mysqlObject *object, char **values, int numberOfChunks)
{
	((struct mysqlQuery*)object->myobject)->values = values;
	((struct mysqlQuery*)object->myobject)->numberOfQueries = numberOfChunks; // Choose one, numberOfChunks or numberOfQueries???
	//printf("NUMBER OF QUERIES IS %d", ((struct mysqlQuery*)object->myobject)->numberOfQueries);
	//pause();
}

char** npFinalFormatChunks(char **buffer, int *numberOfChunks)
{
	char *values = malloc(sizeof(char) * 300000);
	char *formattedValues = malloc(sizeof(char) * 300000); 
	int x = 0;

	for(x = 0; x < *numberOfChunks; x++)
	{
		npFormatChunk(buffer[x], formattedValues, values);
	//	printf("\n CHUNKnum : %d \nformattedValues : %s\n ", x, formattedValues);
		printf("Size of formattedValues : %zu", strlen(formattedValues));

		if(strlen(formattedValues) == 0)
		   (*numberOfChunks)--;
		
		strcpy(buffer[x], formattedValues);
	}		
	
	return buffer;
}
 
char** npGetRetrieveTextTagsValues(char** buffer, int *numberOfChunks, int debug, void* dataRef)
{
	int i = 0;
	//	char* buffer;
	int n = 0;
	pNPrecordTag searchTag = NULL;
	pData data = (pData) dataRef;
	char* valsCopy = malloc(300000);
	int numOfTags = 0;
//	int numberOfChunks = 0;
	
	char* title = malloc(100); //?
	char* desc = malloc(100); //?
	
	//if identical recordID && tableID then replace existing
	printf("\nnpNewRetrieveTextTagsValues");
	//	pause();
	for (i=0; i < kNPrecordTagListMax; i++)
	{
		searchTag = data->io.gl.hud.tags.recordList[i];
		
		if(searchTag != NULL)
		{
			numOfTags++;
			if(debug == 1)
			{
				printf("\nFound New Tag");
				printf("\nsearchTag->id == %d", searchTag->id);
				printf("\nsearchTag->recordID == %d", searchTag->recordID);
				printf("\nsearchTag->tableID == %d", searchTag->tableID);
				printf("\nsearchTag->title == %s", searchTag->title);
				printf("\nsearchTag->description == %s", searchTag->desc);
			}
			
			mysql_escape_string(title, searchTag->title, strlen(searchTag->title));
			mysql_escape_string(desc, searchTag->desc, strlen(searchTag->desc));
			
			//			printf("\ntitle == %s", title);
			//			printf("\ndescription == %s", desc);
			
			
			//format as CSV and fill the buffer, broken into groups for readability
			/*			n += sprintf ((((struct mysqlTable*)object->myobject)->values[0] + n), "%d,%d,%d,\'\"%s\"\',\'\"%s\"\'\n",
			 searchTag->id,					//id is unique to this node
			 searchTag->recordID,		
			 searchTag->tableID,
			 searchTag->title,
			 searchTag->desc
			 );*/
			n += sprintf ((valsCopy + n), "%d,%d,%d,\'\"%s\"\',\'\"%s\"\'\n",
						  searchTag->id,					//id is unique to this node
						  searchTag->recordID,		
						  searchTag->tableID,
						  title,
						  desc
						  );
			
			
			
		}			
	}
	
	//	mysql_escape_string(escaped, buffer, strlen(buffer));
	if(debug == 1)
	{
		//printf("buffer : %s \nSize : %zu", buffer, strlen(buffer));
		//	pause();
	}
	
	if(numOfTags == 0)
	{
		printf("\nNo tags exist");
		return NULL;
	}
	else {
		//chunkValues(buffer, 65536);
		//chunkAllValues(object, buffer, dataRef);
		buffer = npNewChunkAllValues(valsCopy, numberOfChunks, dataRef);
	}
	
	
	//	for(int i = 0; i < *numberOfChunks; i++)
			printf("\nOUTPUTtext :: %s\n", buffer[0]);
	
	//after last group, end the node row using a newline
	//	n += sprintf ((buffer + n), "\n");	
	
	//increment the buffer
	data->io.write[kNPnode] += n;
	
	
	return buffer;
}

void npTraverseTreeDb (void (*nodeFunc)(pNPnode node, void* dataRef), 
					 pNPnode node, void* dataRef)
{
	int i = 0;

	nodeFunc(node, dataRef);

	for (i=0; i < node->childCount; i++)
	{
		// recursively call this function
		npTraverseTree (nodeFunc, node->child[i], dataRef);
		
		// apply passed in function operation to node
		//nodeFunc(node->child[i], dataRef); -- JJ, commented this out because it results in double calls, only need single call below
	}
	//process root node last -- JJ, moved from top so now root really is processed last
	

}


char** npGetNodesValues(char **buffer, int *numberOfChunks, void* dataRef)
{
	pData data = (pData) dataRef;
	pNPnode node = NULL;
	int count = 0;
	int i = 0;
	//	int numberOfChunks;
	
//	char *valsCopy = malloc(10000 * data->map.nodeRootCount); // debug, doesn't account for child nodes
	char *valsCopy = malloc(1000 * data->map.nodeCount);

	if(data->map.nodeCount == 0)
	{
		return NULL;
	}
	
	//char *valsCopy = malloc(3000 * data->map.nodeCount);
	
	
	printf("\nnpRetrieveNodeValues1 : We have this many nodes : %d", data->map.nodeRootCount);
	
	data->io.write[kNPnode] = valsCopy;
	
	count = 0;
	
	//This portion should be in its own function
	for (i=0; i < data->map.nodeRootCount; i++)
	{
		node = data->map.node[i];
		
		if (node->type != kNodeHUD)				// Do something about that not statement later, debug ll						
		{
			count += npWriteNode ((valsCopy + count), node, kNPmapNodeCSV, dataRef);
			
			if (node->childCount)
			{
				count += npMapTraverseTree (valsCopy, node, kNPmapNodeCSV, dataRef);
			}
		}
	}
	
	buffer = npNewChunkAllValues(valsCopy, numberOfChunks, dataRef);
	
	
//	for(int i = 0; i < *numberOfChunks; i++)
//		printf("\nOUTPUT2 :: %s\n", buffer[i]);
	
	//	pause();
	return buffer;
}

int npFormatChunk(char *buffer, char* formattedValues, char* values)
{
	//Free after use
	int count = 0;
	int curs;
	int totSize = strlen(buffer);
	int i = 0;
	memset(values, '\0', 300000);
	memset(formattedValues, '\0', 300000);
	
	printf("\nSize of buffer : %d\n", totSize);
	while(count < totSize)
	{
		count += curs = npSeekToNextLine(buffer);
		
		if(count > totSize) //npSeekToNextLine doesn't check if it has reached end of buffer so this error checking is necessary, debug ll
		{
			printf("\nCOUNT EXCEEDED TOTSIZE!!!!\n");
			break;
		}
			
	//	printf("\ncurs : %d\n", curs);
		for(i = 0; i < curs; i++)
		{
			values[i] = buffer[i];
		}
		values[i+1] = '\0';
		// If not empty
		if(curs > 1) // ??? curs < 500
		{
		strcat(formattedValues, "(");
		strcat(formattedValues, values);
		strcat(formattedValues, ")");
		strcat(formattedValues, ",");
		}

		memset(values, '\0', curs+1);
		
		buffer = &buffer[curs];
	}
	
	formattedValues[strlen(formattedValues)-1] = '\0';
	//printf("\nnpFormatChunk : %s\n", formattedValues);
	
	//	pause();
	return strlen(formattedValues);
}

char** _cdecl npNewChunkAllValues(char* buffer, int *numOfChunks, void* dataRef)
{
	pData data = (pData) dataRef;
	double tester;
	float holder;
	float mod;
	int i;
	char **chunked;
	//	int numOfChunks = 0;
	
	int count = 0;
	//	printf("npFillerFunc : %d", strlen(valsCopy));
	if ( (sizeof(char) * strlen(buffer)) > 1048576)
	{
		//		printf("\nThis is going to be too large : vals[0] : \n%s\n", table->values[0]); // I might have to restructure the data structure
		*numOfChunks =  ((strlen(buffer) * sizeof(char))/(float)65536); // This shouldn't be here, debug ll
		holder = (strlen(buffer)*sizeof(char))/(float)65536;
		mod = modf( holder , &tester);
		
		if(mod > 0)
			(*numOfChunks)++;
		
		printf("NUMBER OF CHUNKS : %d", *numOfChunks);
		
		
		chunked = malloc(sizeof(char*) * *numOfChunks);
		for(i = 0; i < *numOfChunks; i++)
			chunked[i] = malloc(80000 * sizeof(char));
		
		//		((struct mysqlTable*)object->myobject)->values = malloc(sizeof(char*) * numOfChunks);
		//		*(((struct mysqlTable*)object->myobject)->values) = malloc(sizeof(char) * 80000);
		
		for(i = 0 ; i < *numOfChunks; i++)
		{
			count = fixedChunkValues(buffer, chunked[i], 65536);
			//		((struct mysqlTable*)object->myobject)->values[i] = chunked[i];
			buffer = &buffer[count];
			//		printf("\nOUTPUT :: %s\n", chunked[i]);
		}
		
		printf("\nReturned from chunkValues()");
	}
	else {
		*numOfChunks =  1; // This maybe shouldn't be here, debug ll
		//		((struct mysqlTable*)object->myobject)->values = malloc(sizeof(char*) * ((struct mysqlTable*)object->myobject)->numOfChunks);
		//		*(((struct mysqlTable*)object->myobject)->values) = malloc(sizeof(char) * 80000); // Make this dynamic
		//		((struct mysqlTable*)object->myobject)->values[0] = valsCopy;
		chunked = malloc(sizeof(char*) * *numOfChunks);
		chunked[0] = malloc(80000 * sizeof(char));
		chunked[0] = buffer;
		return chunked;
	}
	
	return chunked;
}

int fixedChunkValues(char* buffer, char* chunked, int chunkSize)
{
	int count = 0, curs = 0;
	
	if (strlen(buffer) < chunkSize)
		count = strlen(buffer);
	else 
	{
		
	while(count <= chunkSize)
		count += curs = npSeekToNextLine(&buffer[curs]);
	
		while(buffer[count++] != '\n');
		
	}
			
	memcpy(chunked, buffer, count); // This might be count + curs;
	chunked[count] = '\0';

	return count;
}
