/* -----------------------------------------------------------------------------
*
*  npmysql.h
*
*  ANTz - realtime 3D data visualization tools for the real-world, based on NPE.
*
*  ANTz is hosted at http://openantz.com and NPE at http://neuralphysics.org
*
*  Written in 2010-2012 by Shane Saxon - contact@saxondigital.net
*
*  Please see main.c for a complete list of additional code contributors.
*
*  To the extent possible under law, the author(s) have dedicated all copyright 
*  and related and neighboring rights to this software to the public domain
*  worldwide. This software is distributed without any warranty.
*
*  Released under the CC0 license, which is GPL compatible.
*
*  You should have received a copy of the CC0 Public Domain Dedication along
*  with this software (license file named COPYING.txt). If not, see
*  http://creativecommons.org/publicdomain/zero/1.0/
*
* --------------------------------------------------------------------------- */

#ifndef NPMYSQL_H_
#define NPMYSQL_H_

//#include <mysql.h>
#include "../../npdata.h"
#include "../gl/nptags.h" // Maybe something to relocate
#include "../../data/npmapfile.h"

#include <my_global.h>
#include <mysql.h>
#include "mysqlStructures.h"
#include "mysqlInitFuncs.h"
#include "mysqlSetFuncs.h"
#include "mysqlGetFuncs.h"  // Maybe this should be in mysqlLoadIntoAntz
#include "mysqlGenFuncs.h"
#include "mysqlFormatFuncs.h"
#include "mysqlLoadIntoAntz.h"
//------------------------------------------------------------------------------ 
char*** npNewFilterIDbyInterval(struct dbConnect *connect, char ***buffer, int interval, void* dataRef);

void npTest(void* dataRef);

void npOpenMySQL (struct dbConnect *connection, void* dataRef);

void npNewExecuteMysqlQuery(struct dbConnect *connect,struct mysqlObject *object, void* dataRef);

void* npExecuteQueryBufferMySQL(struct dbConnect *connect, char* buffer, struct dbConnect *connection, int *ErrResult, void* dataRef);

void npCloseMySQL (struct dbConnect *connect, void* dataRef);

void npUpdateMySQL (struct dbConnect *connect, void* dataRef);

int npFreeMysqlQuery(struct dbConnect *connect, struct mysqlObject *object, void* dataRef);

int npFreeMysqlObject(struct dbConnect *connect, struct mysqlObject *object, void* dataRef);

int npFreeMysqlDatabase(struct dbConnect *connect, struct mysqlObject *object, void* dataRef);

int npFreeMysqlTable(struct dbConnect *connect, struct mysqlObject *object, void* dataRef);

int npCreateAntzDatabaseMysql(struct dbConnect *connect, void* dataRef);

int npCreateDatabaseMysql(struct dbConnect *connect, char *name, void* dataRef);

int	npUseAntzDatabaseMysql(struct dbConnect *connect, void* dataRef);

int npUseDatabaseMysql(struct dbConnect *connect, char *name, void* dataRef);

int	npCreateAntzNodeStateTable(struct dbConnect *connect, void* dataRef);

int npCreateTableMysql(struct dbConnect *connect, char *name, char *fields, void* dataRef);

int npCreateAntzTagsTable(struct dbConnect *connect, void* dataRef);

int npInsertAntzTags(struct dbConnect *connect, void* dataRef);

int npInsertIntoTableMysql(struct dbConnect *connect, char *name, char **values, int numChunks, void* dataRef);

int npInsertAntzNodeStates(struct dbConnect *connect, void* dataRef);

int npDropAntzTagsTable(struct dbConnect *connect, void* dataRef);

int npSelectAntzTagsTable(struct dbConnect *connect, void* dataRef);

int npSelectTableMysql(struct dbConnect *connect, struct mysqlObject *query, char *name, void* dataRef);

int npAntzMysqlCtrl(struct dbConnect *connect, int command, int table, void* dataRef);

int npDropAntzTableMysql(struct dbConnect *connect, int tableType, void* dataRef);

int npInsertAntzTableMysql(struct dbConnect *connect, int tableType, void* dataRef);

int npCreateAntzTableMysql(struct dbConnect *connect, int tableType, void* dataRef);

void npNewTest(void* dataRef);

int npAntzDatabaseMysql(struct dbConnect *connect, int command, void* dataRef);

int npAntzMysql(struct dbConnect *connect, int objectType, int tableType, int command, void* dataRef);

int npSelectAntzTableMysql(struct dbConnect *connect, int tableType, void* dataRef);

int npLoadAntzStateFromDatabase(struct dbConnect *connect, void* dataRef);

//int npSaveAntzStateToDatabase(struct dbConnect *connect, void* dataRef);
int npSaveAntzStateToDatabase(struct dbConnect *connect, void* dataRef);

int npGetNodeRangeFromMysql(struct dbConnect *connect, char* field, char* from, char* to, void* dataRef);

int npSetMysqlQueryConditionals(struct dbConnect *connect, struct mysqlObject *query, int conditionalType, char* conditional, void* dataRef);

int npGetRootNodesFromMysql(struct dbConnect *connect, void* dataRef);

int npGetNodesWithMysql(struct dbConnect *connect, int fieldType, int fieldValue, void* dataRef);

char** npRetrieveChMapValues(char** buffer, int *numberOfChunks, void* dataRef);

int npCreateAntzChMapTable(struct dbConnect *connect, void* dataRef);

int npSelectAntzChMapTable(struct dbConnect *connect, void* dataRef);

int npDropAntzChMapTable(struct dbConnect *connect, void* dataRef);

int npSelectAntzChannelsTable(struct dbConnect *connect, void* dataRef);

int npInsertAntzChannelsTable(struct dbConnect *connect, void* dataRef);

char** npRetrieveChannelsValues(struct dbConnect *connect, char** buffer, int* numberOfChunks, void* dataRef);

//int npSampleNodeStateTable(struct dbConnect *connect, int mod, void* dataRef);
int npSampleNodeStateTable(struct dbConnect *connect, int mod, void* dataRef);

char*** npGetResultMysql(struct dbConnect *connect, MYSQL_RES *result, char*** buffer, void* dataRef);

//char*** npGetRootNodesIdsFromMysql(struct dbConnect *connect, char*** buffer, int interval);

char*** npGetRootNodesIdsFromMysql(struct dbConnect *connect, char*** buffer, int interval, void* dataRef); // Add buffer

char*** npGetChildNodeFromRecordIdMysql(struct dbConnect *connect, char* recordID, void* dataRef);

char*** npFilterIDbyInterval(struct dbConnect *connect, char ***buffer, int interval, void* dataRef);

//int npLoadRootNodes(char*** buffer, void* dataRef);
int npLoadRootNodes(struct dbConnect *connect, char*** buffer, void* dataRef);


int npGetChildren(struct dbConnect *connect, char*** buffer, void* dataRef);


#endif

